using Microsoft.Extensions.Caching.Memory;
using System.Net;
using System.Text.Json;
using Amazon.Lambda.APIGatewayEvents;
using Amazon.Lambda.Core;

// Assembly attribute to enable the Lambda function's JSON input to be converted into a .NET class.
[assembly: LambdaSerializer(typeof(Amazon.Lambda.Serialization.SystemTextJson.DefaultLambdaJsonSerializer))]

namespace MemoryCacheInALambdaFunction;

public class Function
{
    private readonly MemoryCache cache;
    private readonly string functionId;
    private readonly string functionCreationTime; 
    private readonly Random random;
    public Function()
    {
        cache = new MemoryCache(new MemoryCacheOptions());
        functionId = Guid.NewGuid().ToString().Substring(0,8); 
        functionCreationTime = DateTime.Now.ToString("HH:mm:ss.fff");
        random = new Random();
    }

    public APIGatewayHttpApiV2ProxyResponse FunctionHandler(APIGatewayHttpApiV2ProxyRequest request, ILambdaContext context)
    {
        Console.WriteLine(JsonSerializer.Serialize(request)); // not needed, but great for seeing what a request looks like
        string itemId = request.QueryStringParameters["itemId"];
        Console.WriteLine($"Got itemId: {itemId}");

        ResponseBody responseBody = new ResponseBody() {
            FunctionId = functionId,
            FunctionCreationTime = functionCreationTime,
            ItemId = itemId
        };  

        if (cache.TryGetValue(itemId, out string quantity))
        {
            responseBody.Quantity = quantity;
            responseBody.FromCache = true;
        }
        else
        {   
            quantity = random.Next(1000).ToString(); // simulate a database call
            responseBody.Quantity = quantity;
            responseBody.FromCache = false;
            cache.Set(itemId, quantity, TimeSpan.FromSeconds(10));
        }
        
        var response = new APIGatewayHttpApiV2ProxyResponse
        {
            StatusCode = (int)HttpStatusCode.OK,
            Body = JsonSerializer.Serialize(responseBody),
            Headers = new Dictionary<string, string> { { "Content-Type", "application/json" } }
        };

        return response;
    }
}

public class ResponseBody
{
    public string FunctionId { get; set; }
    public string FunctionCreationTime { get; set; } 
    public string ItemId { get; set; }  
    public string Quantity { get; set; }
    public bool FromCache { get; set; }   
}